package main

import (
	"log"
	"os"
	"os/user"
	"path"
	"path/filepath"
	"regexp"
	"strings"

	"github.com/jessevdk/go-flags"
	"gopkg.in/yaml.v3"
)

const exitCodeParseArgumentError = 2

type configPathsStruct struct {
	current string
	user    string
	main    string
}

var (
	configPaths = configPathsStruct{
		main: "/etc/requirements-manager/config.yaml",
	}
)

// Exporting interface instead of struct
//type Configuration interface{}

// Configuration
type Configuration struct {
	Path string `yaml:"path"`

	Locale struct {
		DateFormat string `yaml:"date-format"`
	} `yaml:"Locale"`

	Table struct {
		Columns []string `yaml:"columns"`
	} `yaml:"table"`

	Overview struct {
		Template string `yaml:"template"`
	} `yaml:"overview"`

	Requirement struct {
		Template string `yaml:"template"`
	} `yaml:"requirement"`
}

func fileExists(name string) (found bool) {
	if f, err := os.Stat(name); err == nil {
		return f.Mode().IsRegular()
	}
	return false
}

// NewConfiguration
func NewConfiguration() (*flags.Command, *commandLineOptions, *Configuration) {

	arguments := new(commandLineOptions)
	p := flags.NewParser(arguments, flags.Default)
	_, err := p.Parse()
	if err != nil {
		os.Exit(-1)
	}

	current, err := os.Getwd()
	if err != nil {
		panic(err)
	}

	filename := path.Clean(current + "/config.yaml")

	cfg := Configuration{}

	usr, err := user.Current()
	if err == nil {
		configPaths.user = usr.HomeDir + "/.config/requirements-manager/config.yaml"
	}

	switch {
	case fileExists(filename):
		readFile(&cfg, filename)
	case err == nil && fileExists(current):
		readFile(&cfg, current)
	case fileExists(configPaths.user):
		readFile(&cfg, configPaths.user)
	case fileExists(configPaths.main):
		readFile(&cfg, configPaths.main)
	}

	//readEnv(&cfg)

	if arguments.Path != "" {
		cfg.Path = arguments.Path
	}

	if arguments.DateFormat != "" {
		cfg.Locale.DateFormat = arguments.DateFormat
	}

	activeCommand := p.Command.Active

	switch activeCommand.Name {

	case "version":

	case "overview":
		subcommand := activeCommand.Active
		switch subcommand.Name {
		case "print":

			if len(arguments.Overview.Print.Columns) > 0 {
				cfg.Table.Columns = validateColumns(arguments.Overview.Print.Columns)
			}

			if arguments.Overview.Print.TemplatePath != "" {
				p := arguments.Overview.Print.TemplatePath

				current, err := os.Getwd()
				if err != nil {
					log.Println(err)
				}

				if !filepath.IsAbs(p) {
					p = path.Clean(current + "/" + p)
				}

				template, err := os.ReadFile(p)
				if err != nil {
					printErrorAndExit(exitCodeCatchAll,
						"the specified template", arguments.Overview.Print.TemplatePath)
				}

				cfg.Overview.Template = convertTemplateImages(string(template), path.Dir(p))
			} else {
				cfg.Overview.Template = defaultOverviewTemplate
			}
		}
	case "requirements":
		subcommand := activeCommand.Active
		switch subcommand.Name {
		case "add":
			cfg.Requirement.Template = defaultNewRequirementTemplate

		case "print":
			if len(arguments.Requirements.Print.Columns) > 0 {
				cfg.Table.Columns = validateColumns(arguments.Requirements.Print.Columns)
			}
		}
	}

	return p.Command.Active, arguments, &cfg
}

func convertTemplateLogo(content string, absolute string) string {
	todoRegEx := regexp.MustCompile(`(?m)^(?P<match>logo:\s*"?(?P<path>[^"\n]*)"?\s*)$`)

	matches := todoRegEx.FindAllStringSubmatch(content, -1)
	if matches == nil {
		return content
	}

	for _, match := range matches {
		result := make(map[string]string)
		for i, name := range todoRegEx.SubexpNames() {
			if i != 0 && name != "" {
				result[name] = match[i]
			}
		}

		if filepath.IsAbs(result["path"]) {
			continue
		}

		path := path.Clean(absolute + "/" + result["path"])
		content = strings.Replace(content, result["match"], "logo: \""+path+"\"", -1)

	}

	return content
}
func convertTemplateLatexLogo(content string, absolute string) string {
	todoRegEx := regexp.MustCompile(`(?m)(?P<match>\\includegraphics[^{]*\{(?P<path>[^}]*)\})`)

	matches := todoRegEx.FindAllStringSubmatch(content, -1)
	if matches == nil {
		return content
	}

	for _, match := range matches {
		result := make(map[string]string)
		for i, name := range todoRegEx.SubexpNames() {
			if i != 0 && name != "" {
				result[name] = match[i]
			}
		}

		if filepath.IsAbs(result["path"]) {
			continue
		}

		path := path.Clean(absolute + "/" + result["path"])
		a := strings.Replace(result["match"], result["path"], path, -1)

		content = strings.Replace(content, result["match"], a, -1)

	}

	return content
}

func convertTemplateImages(content string, absolute string) string {
	content = convertTemplateLogo(content, absolute)
	content = convertTemplateLatexLogo(content, absolute)

	return content

}

func validateColumns(columns []string) []string {

	//for i, column := range columns {
	//	columns[i] = printer.Sprintf(column)
	//}

	// VALIDATE COLUMS NAME

	//
	//for _, name := range config.Requirements.Columns {
	//
	//	field := reflect.Indirect(r).FieldByName(name)
	//	if field == (reflect.Value{}) {
	//		printError("the specified column " + name + " for the table does not exist")
	//		continue

	//	}

	return columns

}

func readFile(cfg *Configuration, filename string) {
	f, err := os.Open(filename)
	if err != nil {
		printErrorAndExit(2, err.Error())
	}

	defer f.Close()

	decoder := yaml.NewDecoder(f)
	err = decoder.Decode(cfg)
	if err != nil {
		printErrorAndExit(2, err.Error())
	}
}
