package main

import (
	"html/template"
	"io/ioutil"
	"os"
	"path"
	"path/filepath"
	"regexp"
	"sort"
	"strings"
	"time"
)

type Dataset struct {
	Config        *Configuration
	Meta          string
	Tasks         string
	Items         string
	Requirements  string
	Created       time.Time
	CreatedFormat string
}

func PrintOverview(config *Configuration, arguments *commandLineOptions) error {

	err, pageData := GetPageData(config.Path)
	if err != nil {
		return err
	}

	err, meta := buildRequirements(config, pageData)
	if err != nil {
		return err
	}

	err, tasks := buildTasksTable(config, pageData, false)
	if err != nil {
		return err
	}

	err, items := buildItemOverviewTable(config, pageData, false)
	if err != nil {
		return err
	}

	t, err := template.New("overview").Parse(config.Overview.Template)
	if err != nil {
		return err
	}

	requirements := ""

	keys := make([]string, 0, len(pageData))
	for k := range pageData {
		keys = append(keys, k)
	}

	sort.Strings(keys)

	for _, k := range keys {
		p := pageData[k]
		requirements = requirements + getAdjustedContent(p.Absolute)
	}

	d := new(Dataset)
	d.Meta = meta
	d.Tasks = tasks
	d.Config = config
	d.Items = items
	d.Requirements = requirements
	d.Created = time.Now()
	d.CreatedFormat = time.Now().Format(config.Locale.DateFormat)
	//d.requirement = requirement

	if arguments.Overview.Print.Output == "" {
		err = t.Execute(os.Stdout, d)
		if err != nil {
			return err
		}
	}

	outputName := arguments.Overview.Print.Output

	if arguments.Overview.Print.Format == "pdf" {
		file, err := ioutil.TempFile(os.TempDir(), "prefix")
		if err != nil {
			printErrorAndExit(2, "A temporary file cannot be created", err.Error())
		}
		defer os.Remove(file.Name())

		err = t.Execute(file, d)
		if err != nil {
			return err
		}

		convertToPDF(file.Name(), outputName)

	} else {

		file, err := os.Create(outputName)
		if err != nil {
			printErrorAndExit(2, "The output file cannot be written", err.Error())
		}

		// close fo on exit and check for its returned error
		defer func() {
			if err := file.Close(); err != nil {
				printErrorAndExit(2, "The output file cannot be written", err.Error())
			}
		}()

		err = t.Execute(file, d)
		if err != nil {
			return err
		}

	}

	return nil

}

func getAdjustedContent(absolute string) string {

	content, err := os.ReadFile(absolute)
	if err != nil {
		printError("The file cannot be read", absolute)
		return ""
	}

	s := string(content)
	a := strings.Index(s, "\n---")
	if a == -1 {
		return s
	}

	s = s[a+4 : len(s)-1]
	b := strings.Index(s, "\n...")

	if b == -1 {
		b = strings.Index(s, "\n---")
	}

	if b > -1 {
		s = s[b+4 : len(s)-1]
	}

	s = convertImages(s, path.Dir(absolute))

	return s
}

func convertImages(content string, absolute string) string {

	todoRegEx := regexp.MustCompile(`(?P<match>\!\[(?P<label>[^]]*)\]\((?P<path>[^)]*)\))`)

	matches := todoRegEx.FindAllStringSubmatch(content, -1)
	if matches == nil {
		return content
	}

	for _, match := range matches {
		result := make(map[string]string)
		for i, name := range todoRegEx.SubexpNames() {
			if i != 0 && name != "" {
				result[name] = match[i]
			}
		}

		if filepath.IsAbs(result["path"]) {
			continue
		}

		path := path.Clean(absolute + "/" + result["path"])
		content = strings.Replace(content, result["match"], "!["+result["label"]+"]("+path+")", -1)

	}

	return content
}
