package main

import (
	"bytes"
	"fmt"
	"html/template"
	"os"
	"path"
	"reflect"
	"time"

	"github.com/olekukonko/tablewriter"
)

type requirement struct {
	ToDos    []task
	Absolute string
	File     string
	Items    []Item `yaml:"items"`

	ID         string    `yaml:"ID"`
	References []string  `yaml:"References"`
	Type       string    `yaml:"Type"` // Display, Functional, Performance, Printing, Report, Testing and Validate
	Title      string    `yaml:"Title"`
	Alias      []string  `yaml:"Alias"`
	Keywords   []string  `yaml:"Keywords"`
	Author     string    `yaml:"Author"`
	Status     string    `yaml:"Status"`
	Complexity string    `yaml:"Complexity"`
	Difficulty string    `yaml:"Difficulty"` // Low, Medium, High
	Priority   string    `yaml:"Priority"`   // Low, Medium, High
	Version    string    `yaml:"Version"`
	Milestone  string    `yaml:"Milestone"`
	Estimation int       `yaml:"Estimation"` // 0, 1, 2, 3, 5, 8, 13, 20, 40, 100
	Source     string    `yaml:"Source"`
	Created    time.Time `yaml:"Created"`
	LastUpdate time.Time `yaml:"Last Update"`
}

type addRequirementDataset struct {
	ID string
}

func addRequirement(config *Configuration, arguments *commandLineOptions) error {

	if arguments.Requirements.Add.ID == "" {
		printErrorAndExit(2, "the id must not be empty")
	}

	p := path.Join(arguments.Path, arguments.Requirements.Add.ID+".md")
	if fileExists(p) {
		printErrorAndExit(2, "the request with id already exists", arguments.Requirements.Add.ID, p)
	}

	t, err := template.New("requirement").Parse(config.Requirement.Template)
	if err != nil {
		return err
	}

	d := new(addRequirementDataset)
	d.ID = arguments.Requirements.Add.ID

	f, err := os.Create(p)
	if err != nil {
		return err
	}

	defer f.Close()

	err = t.Execute(f, d)
	if err != nil {
		return err
	}

	return nil
}

func translateRequirementHeaders(columns []string) []string {

	result := []string{}

	for _, column := range columns {
		result = append(result, printer.Sprintf(column))
	}

	return result

}

func buildRequirements(config *Configuration, pageMap map[string]requirement) (error, string) {

	buf := new(bytes.Buffer)

	table := tablewriter.NewWriter(buf)

	translateMetaColumns := translateRequirementHeaders(config.Table.Columns)

	table.SetHeader(translateMetaColumns)
	table.SetBorders(tablewriter.Border{Left: true, Top: false, Right: true, Bottom: false})
	table.SetCenterSeparator("|")

	var tableData [][]string

	for _, pd := range pageMap {

		r := reflect.ValueOf(pd)

		var row []string
		found := false
		for _, name := range config.Table.Columns {

			field := reflect.Indirect(r).FieldByName(name)
			if field == (reflect.Value{}) {
				row = append(row, "")
				continue
			}

			t := field.Type()

			switch t.Name() {
			case "string":
				found = true
				row = append(row, field.String())
			case "Time":
				found = true
				t := field.Interface().(time.Time)
				if t.IsZero() {
					row = append(row, "—")
				} else {
					row = append(row, field.Interface().(time.Time).Format(config.Locale.DateFormat))
				}
			case "int":
				found = true
				row = append(row, field.String())
			case "float64":
				found = true
				row = append(row, field.String())
			case "nil":
				row = append(row, field.String())
			case "bool":
				found = true
				expr := field.Bool()
				if expr {
					row = append(row, "true")
				} else {
					row = append(row, "false")
				}

			default:
				row = append(row, "")
			}

		}

		if found {
			tableData = append(tableData, row)
		}

	}

	table.AppendBulk(tableData) // Add Bulk Data
	table.SetHeaderAlignment(tablewriter.ALIGN_LEFT)
	table.Render()

	return nil, buf.String()

}

func printRequirements(config *Configuration) error {
	err, pageData := GetPageData(config.Path)
	if err != nil {
		return err
	}

	err, table := buildRequirements(config, pageData)
	fmt.Println(table)
	return err
}
